package java.util;

import java.io.StreamCorruptedException;
import sun.misc.SharedSecrets;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.function.Function;
import java.util.function.BiFunction;
import java.util.function.BiConsumer;
import java.io.Serializable;

public class Hashtable<K, V> extends Dictionary<K, V> implements Map<K, V>, Cloneable, Serializable
{
    private transient Hashtable.Entry<?, ?>[] table;
    private transient int count;
    private int threshold;
    private float loadFactor;
    private transient int modCount;
    private static final long serialVersionUID = 1421746759512286392L;
    private static final int MAX_ARRAY_SIZE = 2147483639;
    private transient volatile Set<K> keySet;
    private transient volatile Set<Map.Entry<K, V>> entrySet;
    private transient volatile Collection<V> values;
    private static final int KEYS = 0;
    private static final int VALUES = 1;
    private static final int ENTRIES = 2;
    
    public Hashtable(int i, float f) {
        this.modCount = 0;
        if (i < 0) {
            throw new IllegalArgumentException("Illegal Capacity: " + i);
        }
        if (f <= 0.0f || Float.isNaN(f)) {
            throw new IllegalArgumentException("Illegal Load: " + f);
        }
        if (i == 0) {
            i = 1;
        }
        this.loadFactor = f;
        this.table = new Hashtable.Entry[i];
        this.threshold = (int)Math.min((float)i * f, 2.1474836E9f);
        return;
    }
    
    public Hashtable(int n) {
        this(n, 0.75f);
    }
    
    public Hashtable() {
        this(11, 0.75f);
    }
    
    public Hashtable(Map<? extends K, ? extends V> map) {
        this(Math.max(2 * map.size(), 11), 0.75f);
        this.putAll(map);
    }
    
    @Override
    public synchronized int size() {
        return this.count;
    }
    
    @Override
    public synchronized boolean isEmpty() {
        return this.count == 0;
    }
    
    @Override
    public synchronized Enumeration<K> keys() {
        return this.getEnumeration(0);
    }
    
    @Override
    public synchronized Enumeration<V> elements() {
        return this.getEnumeration(1);
    }
    
    public synchronized boolean contains(Object obj) {
        if (obj == null) {
            throw new NullPointerException();
        }
        Hashtable.Entry[] table = this.table;
        int length = table.length;
        while (length-- > 0) {
            Hashtable.Entry<Object, Object> next = table[length];
            while (next != null) {
                if (next.value.equals(obj)) {
                    return true;
                }
                next = next.next;
            }
        }
        return false;
    }
    
    @Override
    public boolean containsValue(Object o) {
        return this.contains(o);
    }
    
    @Override
    public synchronized boolean containsKey(Object obj) {
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        Hashtable.Entry<Object, Object> next = table[(hashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                return true;
            }
            next = next.next;
        }
        return false;
    }
    
    @Override
    public synchronized V get(Object obj) {
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        Hashtable.Entry<Object, Object> next = table[(hashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                return next.value;
            }
            next = next.next;
        }
        return null;
    }
    
    protected void rehash() {
        int length = this.table.length;
        Hashtable.Entry[] table = this.table;
        int n = (length << 1) + 1;
        if (n - 2147483639 > 0) {
            if (length == 2147483639) {
                return;
            }
            n = 2147483639;
        }
        Hashtable.Entry[] table2 = new Hashtable.Entry[n];
        ++this.modCount;
        this.threshold = (int)Math.min((float)n * this.loadFactor, 2.1474836E9f);
        this.table = table2;
        while (length-- > 0) {
            int n2;
            for (Hashtable.Entry next = table[length]; next != null; , next = next.next, n2 = (next.hash & Integer.MAX_VALUE) % n, next.next = table2[n2], table2[n2] = next) {}
        }
    }
    
    private void addEntry(int hashCode, K k, V v, int n) {
        ++this.modCount;
        Hashtable.Entry[] array = this.table;
        if (this.count >= this.threshold) {
            this.rehash();
            array = this.table;
            hashCode = k.hashCode();
            n = (hashCode & Integer.MAX_VALUE) % array.length;
        }
        array[n] = new Hashtable.Entry(hashCode, k, v, array[n]);
        ++this.count;
    }
    
    @Override
    public synchronized V put(K obj, V value) {
        if (value == null) {
            throw new NullPointerException();
        }
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                Object value2 = next.value;
                next.value = value;
                return value2;
            }
            next = next.next;
        }
        this.addEntry(hashCode, obj, value, n);
        return null;
    }
    
    @Override
    public synchronized V remove(Object obj) {
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        Hashtable.Entry<Object, Object> entry = null;
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                ++this.modCount;
                if (entry != null) {
                    entry.next = next.next;
                }
                else {
                    table[n] = next.next;
                }
                --this.count;
                Object value = next.value;
                next.value = null;
                return value;
            }
            else {
                entry = next;
                next = next.next;
            }
        }
        return null;
    }
    
    @Override
    public synchronized void putAll(Map<? extends K, ? extends V> map) {
        for (Map.Entry entry : map.entrySet()) {
            this.put(entry.getKey(), entry.getValue());
        }
    }
    
    @Override
    public synchronized void clear() {
        Hashtable.Entry[] table = this.table;
        ++this.modCount;
        int length = table.length;
        while (--length >= 0) {
            table[length] = null;
        }
        this.count = 0;
    }
    
    public synchronized Object clone() {
        try {
            Hashtable<Object, Object> hashtable = (Hashtable)super.clone();
            hashtable.table = new Hashtable.Entry[this.table.length];
            int length = this.table.length;
            while (length-- > 0) {
                hashtable.table[length] = ((this.table[length] != null) ? ((Hashtable.Entry)this.table[length].clone()) : null);
            }
            hashtable.keySet = null;
            hashtable.entrySet = null;
            hashtable.values = null;
            hashtable.modCount = 0;
            return hashtable;
        }
        catch (CloneNotSupportedException cause) {
            throw new InternalError(cause);
        }
    }
    
    @Override
    public synchronized String toString() {
        int n = this.size() - 1;
        if (n == -1) {
            return "{}";
        }
        StringBuilder sb = new StringBuilder();
        Iterator<Map.Entry<K, V>> iterator = this.entrySet().iterator();
        sb.append('{');
        int n2 = 0;
        while (true) {
            Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
            Object key = entry.getKey();
            Object value = entry.getValue();
            sb.append((key == this) ? "(this Map)" : key.toString());
            sb.append('=');
            sb.append((value == this) ? "(this Map)" : value.toString());
            if (n2 == n) {
                break;
            }
            sb.append(", ");
            ++n2;
        }
        return sb.append('}').toString();
    }
    
    private <T> Enumeration<T> getEnumeration(int n) {
        if (this.count == 0) {
            return Collections.emptyEnumeration();
        }
        return new Hashtable.Enumerator(n, false);
    }
    
    private <T> Iterator<T> getIterator(int n) {
        if (this.count == 0) {
            return Collections.emptyIterator();
        }
        return new Hashtable.Enumerator(n, true);
    }
    
    @Override
    public Set<K> keySet() {
        if (this.keySet == null) {
            this.keySet = Collections.synchronizedSet(new Hashtable.KeySet(), this);
        }
        return this.keySet;
    }
    
    @Override
    public Set<Map.Entry<K, V>> entrySet() {
        if (this.entrySet == null) {
            this.entrySet = Collections.synchronizedSet(new Hashtable.EntrySet(), this);
        }
        return this.entrySet;
    }
    
    @Override
    public Collection<V> values() {
        if (this.values == null) {
            this.values = Collections.synchronizedCollection(new Hashtable.ValueCollection(), this);
        }
        return this.values;
    }
    
    @Override
    public synchronized boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof Map)) {
            return false;
        }
        Map<Object, Object> map = (Map)o;
        if (map.size() != this.size()) {
            return false;
        }
        try {
            for (Map.Entry entry : this.entrySet()) {
                Object key = entry.getKey();
                Object value = entry.getValue();
                if (value == null) {
                    if (map.get(key) != null || !map.containsKey(key)) {
                        return false;
                    }
                    continue;
                }
                else {
                    if (!value.equals(map.get(key))) {
                        return false;
                    }
                    continue;
                }
            }
        }
        catch (ClassCastException ex) {
            return false;
        }
        catch (NullPointerException ex2) {
            return false;
        }
        return true;
    }
    
    @Override
    public synchronized int hashCode() {
        int n = 0;
        if (this.count == 0 || this.loadFactor < 0.0f) {
            return n;
        }
        this.loadFactor = -this.loadFactor;
        Hashtable.Entry[] table;
        for (Hashtable.Entry next : table = this.table) {
            while (next != null) {
                n += next.hashCode();
                next = next.next;
            }
        }
        this.loadFactor = -this.loadFactor;
        return n;
    }
    
    @Override
    public synchronized V getOrDefault(Object o, V v) {
        Object value = this.get(o);
        return (null == value) ? v : value;
    }
    
    @Override
    public synchronized void forEach(BiConsumer<? super K, ? super V> biConsumer) {
        Objects.requireNonNull(biConsumer);
        int modCount = this.modCount;
        Hashtable.Entry[] table;
        for (Hashtable.Entry next : table = this.table) {
            while (next != null) {
                biConsumer.accept(next.key, next.value);
                next = next.next;
                if (modCount == this.modCount) {
                    continue;
                }
                throw new ConcurrentModificationException();
            }
        }
    }
    
    @Override
    public synchronized void replaceAll(BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        int modCount = this.modCount;
        Hashtable.Entry[] array;
        for (Hashtable.Entry next : array = (Hashtable.Entry<?, ?>[])this.table) {
            while (next != null) {
                next.value = Objects.requireNonNull(biFunction.apply(next.key, next.value));
                next = next.next;
                if (modCount == this.modCount) {
                    continue;
                }
                throw new ConcurrentModificationException();
            }
        }
    }
    
    @Override
    public synchronized V putIfAbsent(K obj, V value) {
        Objects.requireNonNull(value);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                Object value2 = next.value;
                if (value2 == null) {
                    next.value = value;
                    return value2;
                }
                return value2;
            }
            else {
                next = next.next;
            }
        }
        this.addEntry(hashCode, obj, value, n);
        return null;
    }
    
    @Override
    public synchronized boolean remove(Object obj, Object obj2) {
        Objects.requireNonNull(obj2);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        Hashtable.Entry<Object, Object> entry = null;
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj) && next.value.equals(obj2)) {
                ++this.modCount;
                if (entry != null) {
                    entry.next = next.next;
                }
                else {
                    table[n] = next.next;
                }
                --this.count;
                next.value = null;
                return true;
            }
            else {
                entry = next;
                next = next.next;
            }
        }
        return false;
    }
    
    @Override
    public synchronized boolean replace(K obj, V obj2, V value) {
        Objects.requireNonNull(obj2);
        Objects.requireNonNull(value);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        Hashtable.Entry<Object, Object> next = table[(hashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                if (next.value.equals(obj2)) {
                    next.value = value;
                    return true;
                }
                return false;
            }
            else {
                next = next.next;
            }
        }
        return false;
    }
    
    @Override
    public synchronized V replace(K obj, V value) {
        Objects.requireNonNull(value);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        Hashtable.Entry<Object, Object> next = table[(hashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                Object value2 = next.value;
                next.value = value;
                return value2;
            }
            next = next.next;
        }
        return null;
    }
    
    @Override
    public synchronized V computeIfAbsent(K obj, Function<? super K, ? extends V> function) {
        Objects.requireNonNull(function);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                return next.value;
            }
            next = next.next;
        }
        Object apply = function.apply(obj);
        if (apply != null) {
            this.addEntry(hashCode, obj, apply, n);
            return apply;
        }
        return apply;
    }
    
    @Override
    public synchronized V computeIfPresent(K obj, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        Hashtable.Entry<Object, Object> entry = null;
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                Object apply = biFunction.apply(obj, next.value);
                if (apply == null) {
                    ++this.modCount;
                    if (entry != null) {
                        entry.next = next.next;
                    }
                    else {
                        table[n] = next.next;
                    }
                    --this.count;
                    return apply;
                }
                else {
                    next.value = apply;
                    return apply;
                }
            }
            else {
                entry = next;
                next = next.next;
            }
        }
        return null;
    }
    
    @Override
    public synchronized V compute(K k, BiFunction<? super K, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Hashtable.Entry[] table = this.table;
        int hashCode = k.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        Hashtable.Entry<Object, Object> entry = null;
        while (next != null) {
            if (next.hash == hashCode && Objects.equals(next.key, k)) {
                Object apply = biFunction.apply(k, next.value);
                if (apply == null) {
                    ++this.modCount;
                    if (entry != null) {
                        entry.next = next.next;
                    }
                    else {
                        table[n] = next.next;
                    }
                    --this.count;
                    return apply;
                }
                else {
                    next.value = apply;
                    return apply;
                }
            }
            else {
                entry = next;
                next = next.next;
            }
        }
        Object apply2 = biFunction.apply(k, null);
        if (apply2 != null) {
            this.addEntry(hashCode, k, apply2, n);
            return apply2;
        }
        return apply2;
    }
    
    @Override
    public synchronized V merge(K obj, V v, BiFunction<? super V, ? super V, ? extends V> biFunction) {
        Objects.requireNonNull(biFunction);
        Hashtable.Entry[] table = this.table;
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % table.length;
        Hashtable.Entry<Object, Object> next = table[n];
        Hashtable.Entry<Object, Object> entry = null;
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                Object apply = biFunction.apply(next.value, v);
                if (apply == null) {
                    ++this.modCount;
                    if (entry != null) {
                        entry.next = next.next;
                    }
                    else {
                        table[n] = next.next;
                    }
                    --this.count;
                    return apply;
                }
                else {
                    next.value = apply;
                    return apply;
                }
            }
            else {
                entry = next;
                next = next.next;
            }
        }
        if (v != null) {
            this.addEntry(hashCode, obj, v, n);
            return v;
        }
        return v;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable.Entry next = null;
        synchronized (this) {
            objectOutputStream.defaultWriteObject();
            objectOutputStream.writeInt(this.table.length);
            objectOutputStream.writeInt(this.count);
            for (int i = 0; i < this.table.length; ++i) {
                for (Hashtable.Entry next2 = this.table[i]; next2 != null; next2 = next2.next) {
                    next = new Hashtable.Entry(0, next2.key, next2.value, next);
                }
            }
        }
        while (next != null) {
            objectOutputStream.writeObject(next.key);
            objectOutputStream.writeObject(next.value);
            next = next.next;
        }
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        objectInputStream.defaultReadObject();
        if (this.loadFactor <= 0.0f || Float.isNaN(this.loadFactor)) {
            throw new StreamCorruptedException("Illegal Load: " + this.loadFactor);
        }
        int int1 = objectInputStream.readInt();
        int i = objectInputStream.readInt();
        if (i < 0) {
            throw new StreamCorruptedException("Illegal # of Elements: " + i);
        }
        int max = Math.max(int1, (int)((float)i / this.loadFactor) + 1);
        int a = (int)((float)(i + i / 20) / this.loadFactor) + 3;
        if (a > i && (a & 0x1) == 0x0) {
            --a;
        }
        int min = Math.min(a, max);
        if (min < 0) {
            min = max;
        }
        SharedSecrets.getJavaOISAccess().checkArray(objectInputStream, Map.Entry[].class, min);
        this.table = new Hashtable.Entry[min];
        this.threshold = (int)Math.min((float)min * this.loadFactor, 2.1474836E9f);
        this.count = 0;
        while (i > 0) {
            this.reconstitutionPut(this.table, objectInputStream.readObject(), objectInputStream.readObject());
            --i;
        }
        return;
    }
    
    private void reconstitutionPut(Hashtable.Entry<?, ?>[] array, K obj, V v) throws StreamCorruptedException {
        if (v == null) {
            throw new StreamCorruptedException();
        }
        int hashCode = obj.hashCode();
        int n = (hashCode & Integer.MAX_VALUE) % array.length;
        Hashtable.Entry<?, ?> next = array[n];
        while (next != null) {
            if (next.hash == hashCode && next.key.equals(obj)) {
                throw new StreamCorruptedException();
            }
            next = next.next;
        }
        array[n] = new Hashtable.Entry(hashCode, obj, v, array[n]);
        ++this.count;
        return;
    }
    
    static Iterator access$100(Hashtable hashtable, int n) {
        return hashtable.getIterator(n);
    }
    
    private class KeySet extends AbstractSet<K>
    {
        @Override
        public Iterator<K> iterator() {
            return Hashtable.access$100(Hashtable.this, 0);
        }
        
        @Override
        public int size() {
            return Hashtable.this.count;
        }
        
        @Override
        public boolean contains(Object o) {
            return Hashtable.this.containsKey(o);
        }
        
        @Override
        public boolean remove(Object o) {
            return Hashtable.this.remove(o) != null;
        }
        
        @Override
        public void clear() {
            Hashtable.this.clear();
        }
    }
    
    private class EntrySet extends AbstractSet<Map.Entry<K, V>>
    {
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return Hashtable.access$100(Hashtable.this, 2);
        }
        
        @Override
        public boolean add(Map.Entry<K, V> entry) {
            return super.add(entry);
        }
        
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object key = entry.getKey();
            Hashtable.Entry[] access$400 = Hashtable.this.table;
            int hashCode = key.hashCode();
            Hashtable.Entry<Object, Object> next = access$400[(hashCode & Integer.MAX_VALUE) % access$400.length];
            while (next != null) {
                if (next.hash == hashCode && next.equals(entry)) {
                    return true;
                }
                next = next.next;
            }
            return false;
        }
        
        @Override
        public boolean remove(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            Object key = entry.getKey();
            Hashtable.Entry[] access$400 = Hashtable.this.table;
            int hashCode = key.hashCode();
            int n = (hashCode & Integer.MAX_VALUE) % access$400.length;
            Hashtable.Entry<Object, Object> next = access$400[n];
            Hashtable.Entry<Object, Object> entry2 = null;
            while (next != null) {
                if (next.hash == hashCode && next.equals(entry)) {
                    Hashtable.this.modCount++;
                    if (entry2 != null) {
                        entry2.next = next.next;
                    }
                    else {
                        access$400[n] = next.next;
                    }
                    Hashtable.this.count--;
                    next.value = null;
                    return true;
                }
                else {
                    entry2 = next;
                    next = next.next;
                }
            }
            return false;
        }
        
        @Override
        public int size() {
            return Hashtable.this.count;
        }
        
        @Override
        public void clear() {
            Hashtable.this.clear();
        }
    }
    
    private class ValueCollection extends AbstractCollection<V>
    {
        @Override
        public Iterator<V> iterator() {
            return Hashtable.access$100(Hashtable.this, 1);
        }
        
        @Override
        public int size() {
            return Hashtable.this.count;
        }
        
        @Override
        public boolean contains(Object o) {
            return Hashtable.this.containsValue(o);
        }
        
        @Override
        public void clear() {
            Hashtable.this.clear();
        }
    }
    
    private static class Entry<K, V> implements Map.Entry<K, V>
    {
        final int hash;
        final K key;
        V value;
        Hashtable.Entry<K, V> next;
        
        protected Entry(int hash, K key, V value, Hashtable.Entry<K, V> next) {
            this.hash = hash;
            this.key = key;
            this.value = value;
            this.next = next;
        }
        
        @Override
        protected Object clone() {
            return new Hashtable.Entry(this.hash, this.key, this.value, (this.next == null) ? null : ((Hashtable.Entry)this.next.clone()));
        }
        
        @Override
        public K getKey() {
            return this.key;
        }
        
        @Override
        public V getValue() {
            return this.value;
        }
        
        @Override
        public V setValue(V value) {
            if (value == null) {
                throw new NullPointerException();
            }
            Object value2 = this.value;
            this.value = value;
            return value2;
        }
        
        @Override
        public boolean equals(Object o) {
            if (!(o instanceof Map.Entry)) {
                return false;
            }
            Map.Entry<Object, Object> entry = (Map.Entry)o;
            return ((this.key != null) ? this.key.equals(entry.getKey()) : (entry.getKey() == null)) && ((this.value != null) ? this.value.equals(entry.getValue()) : (entry.getValue() == null));
        }
        
        @Override
        public int hashCode() {
            return this.hash ^ Objects.hashCode(this.value);
        }
        
        @Override
        public String toString() {
            return this.key.toString() + "=" + this.value.toString();
        }
    }
    
    private class Enumerator<T> implements Enumeration<T>, Iterator<T>
    {
        Hashtable.Entry<?, ?>[] table;
        int index;
        Hashtable.Entry<?, ?> entry;
        Hashtable.Entry<?, ?> lastReturned;
        int type;
        boolean iterator;
        protected int expectedModCount;
        
        Enumerator(int type, boolean iterator) {
            this.table = Hashtable.this.table;
            this.index = this.table.length;
            this.expectedModCount = Hashtable.this.modCount;
            this.type = type;
            this.iterator = iterator;
        }
        
        @Override
        public boolean hasMoreElements() {
            Hashtable.Entry<Object, Object> entry = this.entry;
            int index = this.index;
            for (Hashtable.Entry[] table = this.table; entry == null && index > 0; entry = table[--index]) {}
            this.entry = entry;
            this.index = index;
            return entry != null;
        }
        
        @Override
        public T nextElement() {
            Hashtable.Entry<Object, Object> entry = this.entry;
            int index = this.index;
            for (Hashtable.Entry[] table = this.table; entry == null && index > 0; entry = table[--index]) {}
            this.entry = entry;
            this.index = index;
            if (entry != null) {
                Hashtable.Entry<?, ?> entry2 = this.entry;
                this.lastReturned = entry2;
                Hashtable.Entry<Object, Object> entry3 = entry2;
                this.entry = entry3.next;
                return (this.type == 0) ? entry3.key : ((this.type == 1) ? entry3.value : entry3);
            }
            throw new NoSuchElementException("Hashtable Enumerator");
        }
        
        @Override
        public boolean hasNext() {
            return this.hasMoreElements();
        }
        
        @Override
        public T next() {
            if (Hashtable.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            return this.nextElement();
        }
        
        @Override
        public void remove() {
            if (!this.iterator) {
                throw new UnsupportedOperationException();
            }
            if (this.lastReturned == null) {
                throw new IllegalStateException("Hashtable Enumerator");
            }
            if (Hashtable.this.modCount != this.expectedModCount) {
                throw new ConcurrentModificationException();
            }
            synchronized (Hashtable.this) {
                Hashtable.Entry[] access$400 = Hashtable.this.table;
                int n = (this.lastReturned.hash & Integer.MAX_VALUE) % access$400.length;
                Hashtable.Entry<Object, Object> next = access$400[n];
                Hashtable.Entry<Object, Object> entry = null;
                while (next != null) {
                    if (next == this.lastReturned) {
                        Hashtable.this.modCount++;
                        ++this.expectedModCount;
                        if (entry == null) {
                            access$400[n] = next.next;
                        }
                        else {
                            entry.next = next.next;
                        }
                        Hashtable.this.count--;
                        this.lastReturned = null;
                        return;
                    }
                    else {
                        entry = next;
                        next = next.next;
                    }
                }
                throw new ConcurrentModificationException();
            }
        }
    }
}
